//
//  UIImage+Extension.m
//  SLAiELTS
//
//  Created by Gusont on 2023/3/10.
//

#import "UIImage+Extension.h"

@implementation UIImage (Extension)

+ (NSData *)compressImage:(UIImage *)image {
    CGFloat scale = image.size.height / image.size.width;
    CGSize size = CGSizeMake(240, 240*scale);
//    DYJLog(@"imagesize:%@ oldimage:%ld", NSStringFromCGSize(size), (unsigned long)UIImagePNGRepresentation(image).length/1024);
    UIGraphicsBeginImageContextWithOptions(size, YES, 0);
    [image drawInRect:CGRectMake(0, 0, size.width, size.height)];
    UIImage *smallImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    NSData *imagedata = UIImagePNGRepresentation(smallImage);
//    DYJLog(@"newImage:%ld", (unsigned long)imagedata.length/1024);
    CGFloat compereScale = 0.9;
    // 将图片压至小于200kb
    while (200*1024.0 < imagedata.length) {
        imagedata = UIImageJPEGRepresentation(smallImage, compereScale);
        compereScale -= 0.1;
//        DYJLog(@"scaleimage:%ld", (unsigned long)imagedata.length/1024);
    }
    return imagedata;
}


//纯色图
+ (UIImage *)imageWithColor:(UIColor *)color size:(CGSize)size
{
    CGRect rect = CGRectMake(0.0f, 0.0f, size.width, size.height);
    UIGraphicsBeginImageContext(rect.size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, [color CGColor]);
    CGContextFillRect(context, rect);
    UIImage*theImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return theImage;
}

//纯色图-圆角
+ (UIImage *)imageWithColor:(UIColor *)color
                       size:(CGSize)size
               cornerRadius:(CGFloat)cornerRadius
{
    CGRect rect = CGRectMake(0.0f, 0.0f, size.width, size.height);
    UIGraphicsBeginImageContext(rect.size);
    if (cornerRadius > 0) {
        UIBezierPath *path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(0, 0, size.width, size.height) cornerRadius:cornerRadius];
        [path addClip];
    }
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, [color CGColor]);
    CGContextFillRect(context, rect);
    UIImage*theImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return theImage;
}

//渐变图
+ (UIImage *)gradientImageSize:(CGSize)size
                    startPoint:(CGPoint)startPoint
                      endPoint:(CGPoint)endPoint
                        colors:(NSArray <UIColor *> *)colorStrs
{
    return [self gradientImageSize:size startPoint:startPoint endPoint:endPoint colors:colorStrs cornerRadius:0];
}

//渐变图-圆角
+ (UIImage *)gradientImageSize:(CGSize)size
                    startPoint:(CGPoint)startPoint
                      endPoint:(CGPoint)endPoint
                        colors:(NSArray <UIColor *> *)colorStrs
                  cornerRadius:(CGFloat)cornerRadius
{
    if (!colorStrs || colorStrs.count <= 0) {
        return nil;
    }
    NSMutableArray *gradientColors = [NSMutableArray arrayWithCapacity:colorStrs.count + 1];
    for (int i = 0; i < colorStrs.count; i++) {
        UIColor *color = colorStrs[i];
        if (color) {
            [gradientColors addObject:(id)color.CGColor];
        }
    }
    
    UIGraphicsBeginImageContextWithOptions(size, NO, 0.0);
    if (cornerRadius > 0) {
        UIBezierPath *path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(0, 0, size.width, size.height) cornerRadius:cornerRadius];
        [path addClip];
    }
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    CGColorSpaceRef rgb = CGColorSpaceCreateDeviceRGB();
    CGGradientRef gradient = CGGradientCreateWithColors(rgb, (__bridge CFArrayRef)gradientColors, NULL);
    CGContextDrawLinearGradient(context, gradient, startPoint, endPoint, 0);
    CGGradientRelease(gradient);
    CGColorSpaceRelease(rgb);
    
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return image;
}

// 字符串转图片
+ (UIImage *)imageFromBase64String:(NSString *)string
{
    if (string) {
        NSString *imgStr = [string componentsSeparatedByString:@";base64,"].lastObject;
        // NSString --> NSData
        NSData *data=[[NSData alloc] initWithBase64EncodedString:imgStr options:NSDataBase64DecodingIgnoreUnknownCharacters];
        // NSData --> UIImage
        UIImage *image = [UIImage imageWithData:data];
        return image;
    }
    return nil;
}

#pragma mark - 压缩图片到指定大小(单位KB)
+ (NSData *)resetSizeOfImageData:(UIImage *)sourceImage
                         maxSize:(NSInteger)maxSize
{
    //先判断当前质量是否满足要求，不满足再进行压缩
    __block NSData *finallImageData = UIImageJPEGRepresentation(sourceImage,1.0);
    
    NSLog(@"before:%ldKB(%0.2f,%0.2f,%0.1f)", finallImageData.length/1024, sourceImage.size.width,sourceImage.size.height,sourceImage.scale);
    
    NSUInteger sizeOrigin   = finallImageData.length;
    NSUInteger sizeOriginKB = sizeOrigin / 1024;
    
    if (sizeOriginKB <= maxSize) {
        return finallImageData;
    }
    
    //获取原图片宽高比
    CGFloat sourceImageAspectRatio = sourceImage.size.width/sourceImage.size.height;
    //先调整分辨率
    CGSize defaultSize = CGSizeMake(1024, 1024/sourceImageAspectRatio);
    UIImage *newImage = [self newSizeImage:defaultSize image:sourceImage];
    
    finallImageData = UIImageJPEGRepresentation(newImage,1.0);
    
    //保存压缩系数
    NSMutableArray *compressionQualityArr = [NSMutableArray array];
    CGFloat avg   = 1.0/500;
    CGFloat value = avg;
    for (int i = 500; i >= 1; i--) {
        value = i*avg;
        [compressionQualityArr addObject:@(value)];
    }
    
    /*
     调整大小
     说明：压缩系数数组compressionQualityArr是从大到小存储。
     */
    //思路：使用二分法搜索
    __block NSData *canCompressMinData = [NSData data];//当无法压缩到指定大小时，用于存储当前能够压缩到的最小值数据。
    [self halfFuntion:compressionQualityArr image:newImage sourceData:finallImageData maxSize:maxSize resultBlock:^(NSData *finallData, NSData *tempData) {
        finallImageData = finallData;
        canCompressMinData = tempData;
    }];
    
    
    //如果还是未能压缩到指定大小，则进行降分辨率
    while (finallImageData.length == 0) {
        //每次降100分辨率
        CGFloat reduceWidth = 100.0;
        CGFloat reduceHeight = 100.0/sourceImageAspectRatio;
        if (defaultSize.width-reduceWidth <= 0 || defaultSize.height-reduceHeight <= 0) {
            break;
        }
        defaultSize = CGSizeMake(defaultSize.width-reduceWidth, defaultSize.height-reduceHeight);
        UIImage *image = [self newSizeImage:defaultSize
                                      image:[UIImage imageWithData:UIImageJPEGRepresentation(newImage,[[compressionQualityArr lastObject] floatValue])]];
        [self halfFuntion:compressionQualityArr image:image sourceData:UIImageJPEGRepresentation(image,1.0) maxSize:maxSize resultBlock:^(NSData *finallData, NSData *tempData) {
            finallImageData = finallData;
            canCompressMinData = tempData;
        }];
    }
    //如果分辨率已经无法再降低，则直接使用能够压缩的那个最小值即可
    if (finallImageData.length==0) {
        finallImageData = canCompressMinData;
    }
    UIImage *image = [UIImage imageWithData:finallImageData];
    NSLog(@"after:%ldKB(%0.2f,%0.2f,%0.1f)", finallImageData.length/1024, image.size.width,image.size.height,image.scale);
    return finallImageData;
}


#pragma mark 调整图片分辨率/尺寸（等比例缩放）
///调整图片分辨率/尺寸（等比例缩放）
+ (UIImage *)newSizeImage:(CGSize)size
                    image:(UIImage *)sourceImage
{
    CGSize newSize = CGSizeMake(sourceImage.size.width, sourceImage.size.height);
    
    CGFloat tempHeight = newSize.height / size.height;
    CGFloat tempWidth = newSize.width / size.width;
    
    if (tempWidth > 1.0 && tempWidth > tempHeight) {
        newSize = CGSizeMake(sourceImage.size.width / tempWidth, sourceImage.size.height / tempWidth);
    } else if (tempHeight > 1.0 && tempWidth < tempHeight) {
        newSize = CGSizeMake(sourceImage.size.width / tempHeight, sourceImage.size.height / tempHeight);
    }else {
        newSize = size;
    }
    
    //    UIGraphicsBeginImageContext(newSize);
    UIGraphicsBeginImageContextWithOptions(newSize, YES, 0);
    [sourceImage drawInRect:CGRectMake(0,0,newSize.width,newSize.height)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}


#pragma mark 二分法
///二分法，block回调中finallData长度不为零表示最终压缩到了指定的大小，如果为零则表示压缩不到指定大小。tempData表示当前能够压缩到的最小值。
+ (void)halfFuntion:(NSArray *)arr
              image:(UIImage *)image
         sourceData:(NSData *)finallImageData
            maxSize:(NSInteger)maxSize
        resultBlock:(void(^)(NSData *finallData, NSData *tempData))block
{
    NSData *tempData = [NSData data];
    NSUInteger start = 0;
    NSUInteger end = arr.count - 1;
    NSUInteger index = 0;
    
    NSUInteger difference = NSIntegerMax;
    while(start <= end) {
        index = start + (end - start)/2;
        
        finallImageData = UIImageJPEGRepresentation(image,[arr[index] floatValue]);
        
        NSUInteger sizeOrigin = finallImageData.length;
        NSUInteger sizeOriginKB = sizeOrigin / 1024;
        NSLog(@"当前降到的质量：%ld", (unsigned long)sizeOriginKB);
//        NSLog(@"\nstart：%zd\nend：%zd\nindex：%zd\n压缩系数：%lf", start, end, (unsigned long)index, [arr[index] floatValue]);
        
        if (sizeOriginKB > maxSize) {
            start = index + 1;
        } else if (sizeOriginKB < maxSize) {
            if (maxSize-sizeOriginKB < difference) {
                difference = maxSize-sizeOriginKB;
                tempData = finallImageData;
            }
            if (index<=0) {
                break;
            }
            end = index - 1;
        } else {
            break;
        }
    }
    NSData *d = [NSData data];
    if (tempData.length==0) {
        d = finallImageData;
    }
    if (block) {
        block(tempData, d);
    }
    //    return tempData;
}


//解压gif的data为图片数组
+ (UIImage *)decodeGifImageByData:(NSData *)data {
    //获取data资源器，这个可以直接操作图片data
    CGImageSourceRef source = CGImageSourceCreateWithData((__bridge CFDataRef)data, nil);
    //获取图片数量 size_t 类似于无符号int
    size_t count = CGImageSourceGetCount(source);
    UIImage * animationImage;
    
    //如果只有一张则直接解压
    if (count <= 1) {
        animationImage = [[UIImage alloc]initWithData:data];
    }
    else {
        NSMutableArray * imageArray = [NSMutableArray arrayWithCapacity:count];
        NSTimeInterval duration = 0.0f;
        //遍历获取每一张图片
        for (size_t i = 0; i < count; i++) {
            //解析图片拿到图片画笔拿到图片画笔
            CGImageRef imageRef = CGImageSourceCreateImageAtIndex(source, i, NULL);
            duration += [self imageDurationAtIndex:i source:source];
            //scale:图片缩放因子 默认1  orientation:图片绘制方向 默认网上
            [imageArray addObject:[UIImage imageWithCGImage:imageRef scale:[UIScreen mainScreen].scale orientation:UIImageOrientationUp]];
            CGImageRelease(imageRef);
        }
        
        //如果没有抓取到图片播放时间，则按照0.1秒一张的方式来了播放
        if (!duration) {
            duration = (1.0f / 10.0f) * count;
        }
        animationImage = [UIImage animatedImageWithImages:imageArray duration:duration];
    }
    
    
    CFRelease(source);
    return animationImage;
}

//获取每一张图片的持续时间
+ (float)imageDurationAtIndex:(NSUInteger)index source:(CGImageSourceRef)source {
    float imageDuration = 0.1f;
    //获取指定图像的属性信息 如宽、高、持续时间等都在里面 详情参考 CGImageProperties
    CFDictionaryRef cfImageProperties = CGImageSourceCopyPropertiesAtIndex(source, index, nil);
    if (!cfImageProperties) {
        return imageDuration;
    }
    NSDictionary * imageProperties = (__bridge NSDictionary *) cfImageProperties;
    //拿到gif图的属性信息 获取每一帧的时间
    NSDictionary * gifProperties = [imageProperties valueForKey:(NSString *)kCGImagePropertyGIFDictionary];
    NSNumber * delayTime = [gifProperties valueForKey:(NSString *)kCGImagePropertyGIFUnclampedDelayTime];
    if (delayTime != nil) {
        return delayTime.floatValue;
    }
    
    return imageDuration;
}

@end
